#===========================================
# Where Is My Feature 1.7, tons of features are unmapped yet
#===========================================
# CREATE ActorDumps folder in the Data folder before using this

module ActorDumper
  # Configuration (you can move these to a module constants if preferred)
  DEBUG_ACTOR_INDEX = 1  # Second party member (0-based would be 1)
  DUMP_ON_LOAD = true    # Enable/disable dump on save load
  DUMP_ON_BATTLE = true  # Enable/disable dump on battle start
  # Feature code to name mapping (expanded from your weapon dump)
  FEATURE_NAMES = {
    11 => "Element Rate",
    12 => "Debuff Rate", 
    13 => "State Rate",
    14 => "State Resist",
    21 => "Param",
    22 => "Ex-Param",
    23 => "Sp-Param",  # Changed from "Seal Param" to correct name
    31 => "Atk Element",     # Attack Element
    32 => "Atk State",       # Attack State
    33 => "Atk Speed",       # Attack Speed Bonus
    34 => "Atk Times+",      # Additional Attack Times
    41 => "Add Skill Type",
    42 => "Seal Skill Type",
    43 => "Add Skill",
    44 => "Seal Skill",
    51 => "Equip Weapon Type",
    52 => "Equip Armor Type",
    53 => "Lock Equip",
    54 => "Seal Equip",
    55 => "Slot Type",
    61 => "Action Times+",
    62 => "Special Flag",
    63 => "Collapse Effect",
    64 => "Party Ability",
    66 => "Add Ex-Parameter",
    67 => "Party Ex Ability",
    68 => "Battler Ability",
    69 => "Multi Booster",
    70 => "Dummy Enchant",
    71 => "Terrain Booster",
    72 => "Equip Mastery",
    73 => "Skill State Add",
    74 => "Skill Type State Add",
    
    
    
    117 => "EX Category",
    
  }

  # Parameter names (for Add/Seal Param)
  PARAM_NAMES = [
    "Max HP", "Max MP", "Attack", "Defense",
    "Magic Attack", "Magic Defense", "Agility", "Luck"
  ]

  # Ex-Parameter names (same as weapon dump)
  EX_PARAM_NAMES = [
    "HIT (Accuracy)", "EVA (Evasion)", "CRI (Critical Rate)",
    "CEV (Critical Evasion)", "MEV (Magic Evasion)",
    "MRF (Magic Reflection)", "CNT (Counter Attack)",
    "HRG (HP Regeneration)", "MRG (MP Regeneration)",
    "TRG (TP Regeneration)"
  ]
  
  # Special Parameter names (SP-Params)
  SP_PARAM_NAMES = [
    "TGR (Target Rate)",    # 0
    "GRD (Guard Effect)",   # 1
    "REC (Recovery Effect)",# 2
    "PHA (Pharmacology)",   # 3
    "MCR (MP Cost Rate)",   # 4
    "TCR (TP Charge Rate)", # 5
    "PDR (Physical Damage)",# 6
    "MDR (Magical Damage)", # 7
    "FDR (Floor Damage)",   # 8
    "EXR (Experience Rate)" # 9
  ]

# Dump features for a specific actor (second party member)
def self.dump_actor_features(actor, context)
  return unless actor.is_a?(Game_Actor)
  
  features = actor.respond_to?(:all_features) ? actor.all_features : (actor.features rescue [])
  
  output = []
  output << "=== Actor Feature Dump (#{context}) ==="
  output << "Actor: #{actor.name} (ID: #{actor.id})"
  
  features.sort_by { |f| [f.code, f.data_id] }.each do |feature|
    feature_name = FEATURE_NAMES[feature.code] || "Unknown(#{feature.code})"
    data_info = resolve_feature_data(feature)
    
    # New format with full IDs:
    output << "  #{feature_name} (id:#{feature.code}): #{data_info} (data:#{feature.data_id}), Value: #{feature.value}"
  end

  File.open("Data/ActorDumps/actor_#{actor.id}_features.txt", "w") { |f| f.puts output.join("\n") }
end

  def self.resolve_feature_data(feature)
    case feature.code
    # Standard features
    when 11 # Element Rate
      $data_system.elements[feature.data_id] rescue "Element#{feature.data_id}"
      
    when 13, 14, 33, 34 # State-related
      resolve_state_name(feature.data_id)
      
    when 21 # Param
      $data_system.terms.params[feature.data_id] rescue "Param#{feature.data_id}"
      
      when 22 # Ex-Parameter
    if feature.data_id.between?(0, EX_PARAM_NAMES.size - 1)
      EX_PARAM_NAMES[feature.data_id]
    else
      "UnknownExParam_#{feature.data_id}"
    end
    
    when 23 # Sp-Parameter
    if feature.data_id.between?(0, SP_PARAM_NAMES.size - 1)
      SP_PARAM_NAMES[feature.data_id]
    else
      # Try NWFeature mapping if available
      if defined?(NWFeature::SParam) && NWFeature::SParam.const_defined?(feature.data_id)
        NWFeature::SParam.const_get(feature.data_id).to_s
      else
        "SpParam_#{feature.data_id}"
      end
    end
    
    when 31 # Attack Element
    element = $data_system.elements[feature.data_id] rescue nil
    element ? element : "Element#{feature.data_id}"

  when 32 # Attack State
    state = $data_states[feature.data_id] rescue nil
    state ? state.name : "State#{feature.data_id}"

  when 33 # Attack Speed
    "#{feature.value > 0 ? '+' : ''}#{feature.value} Speed"

  when 34 # Attack Times+
    base = feature.value.to_i
    decimal = feature.value - base
    if decimal > 0
      "#{base}+#{decimal} Attacks"
    else
      "#{base} Extra Attack#{base > 1 ? 's' : ''}"
    end
      
    when 51 # Weapon Type
      $data_system.weapon_types[feature.data_id] rescue "WType#{feature.data_id}"
      
    when 52 # Armor Type
      $data_system.armor_types[feature.data_id] rescue "AType#{feature.data_id}"
      
    # NWFeature special cases
    when 68 # Battler Ability
      if defined?(NWFeature::Battler)
	NWFeature::Battler.constants.find { |c| NWFeature::Battler.const_get(c) == feature.data_id } || 
	"UnknownAbility_#{feature.data_id}"
      else
	"BattlerData_#{feature.data_id}"
      end
    
    when 67 # Party Ex Ability
      if defined?(NWFeature::PartyEx)
	NWFeature::PartyEx.constants.find { |c| NWFeature::PartyEx.const_get(c) == feature.data_id } ||
	"UnknownPartyEx_#{feature.data_id}"
      else
	"PartyExData_#{feature.data_id}"
      end
      
      # Add other special cases as needed...
    else
      "Data#{feature.data_id}"
    end
  end

  def self.resolve_state_name(state_id)
    # Yanfly compatibility
    if defined?(Yanfly::State) && Yanfly::State::NAME[state_id]
      return Yanfly::State::NAME[state_id]
    end
    
    # Vanilla fallback
    ($data_states[state_id] && $data_states[state_id].name) || "State#{state_id}"
  end


  # Helper to get the second party member
  def self.second_party_member
  # Check if party has at least 2 members
  if $game_party.members.size > 1
    $game_party.members[1]  # Second member (0-based index)
  else
    p "Party has only #{$game_party.members.size} members!"
    nil
  end
  end
end

class Scene_Load
  alias nw_load_success on_load_success
  def on_load_success
    nw_load_success
    # Verify
    ActorDumper.dump_actor_features($game_party.members[1], "Post-Load")
  end
end

class Game_Troop
  alias actor_dump_original_setup setup
  def setup(troop_id)
    actor_dump_original_setup(troop_id)  # Pass through troop_id
    if ActorDumper::DUMP_ON_BATTLE
      actor = ActorDumper.second_party_member
      ActorDumper.dump_actor_features(actor, "Battle Start") if actor
    end
  end
end
